@@ -1,7 +1,6 @@
 package CGI::Application::Plugin::RunmodeParseKeyword;
-$CGI::Application::Plugin::RunmodeParseKeyword::VERSION = '0.10_01'; # TRIAL
-
-$CGI::Application::Plugin::RunmodeParseKeyword::VERSION = '0.1001';
+$CGI::Application::Plugin::RunmodeParseKeyword::VERSION = '0.15_01'; # TRIAL
+$CGI::Application::Plugin::RunmodeParseKeyword::VERSION = '0.1501';
 use warnings;
 use strict;
 
@@ -11,24 +10,31 @@ CGI::Application::Plugin::RunmodeParseKeyword - Declare runmodes using Parse::Ke
 
 =cut
 
-use base 'Exporter';
 our @EXPORT = qw(runmode errormode startmode);
 use Carp qw(croak);
 use Sub::Name 'subname';
+use Data::Dumper;
 use Parse::Keyword {};
 
+$Carp::Internal{ (__PACKAGE__) }++;
+
 sub import {
     my $caller = caller;
     my $class = shift;
     my %args = @_;
+    my $into = delete $args{into} || $caller;
     my $inv = delete $args{invocant} || '$self';
 
     Parse::Keyword->import( { runmode   => sub { my ($kw) = @_; parse_mode($kw, $inv); } } );
     Parse::Keyword->import( { errormode => sub { my ($kw) = @_; parse_mode($kw, $inv); } } );
     Parse::Keyword->import( { startmode => sub { my ($kw) = @_; parse_mode($kw, $inv); } } );
 
-    local $Exporter::ExportLevel = 1;
-    $class->SUPER::import(@EXPORT);
+    for my $e(@EXPORT) {
+        my $fn = $into . '::' . $e;
+        no strict 'refs';
+        *$fn = \&$e;
+        *$fn if 0;
+    }
 }
 
 sub runmode { @_ ? $_[0] : () }
@@ -98,7 +104,7 @@ sub parse_mode {
     my $name = parse_name();
     my $sig  = parse_signature($invocant);
     my $attr = parse_attributes();
-    my $body = parse_body($sig);
+    my $body = parse_body($sig) or ($@ and die);
 
     if (defined $name) {
         my $full_name = join('::', compiling_package, $name);
@@ -150,7 +156,8 @@ sub parse_signature {
     my ($invocant_name) = @_;
     lex_read_space;
 
-    return unless lex_peek eq '(';
+    my @vars = ({ index => 0, name => $invocant_name });
+    return \@vars unless lex_peek eq '(';
 
     my @attr = ();
 
@@ -159,14 +166,13 @@ sub parse_signature {
 
     if (lex_peek eq ')') {
         lex_read;
-        return;
+        return \@vars;
     }
 
     my $seen_slurpy;
-    my @vars = ({ index => 0, name => $invocant_name });
     while ((my $sigil = lex_peek) ne ')') {
         my $var = {};
-        die "syntax error"
+        die "syntax error: expected ')' but found '$sigil'"
             unless $sigil eq '$' || $sigil eq '@' || $sigil eq '%';
         die "Can't declare parameters after a slurpy parameter"
             if $seen_slurpy;
@@ -197,8 +203,9 @@ sub parse_signature {
 
         push @vars, $var;
 
-        die "syntax error"
-            unless lex_peek eq ')' || lex_peek eq ',';
+        my $c = lex_peek;
+        die "syntax error: expected ')' or ',' but found '$c'"
+            unless $c eq ')' || $c eq ',';
 
         if (lex_peek eq ',') {
             lex_read;
@@ -283,7 +290,9 @@ sub parse_body {
 
     lex_read_space;
 
-    if (lex_peek eq '{') {
+    my $c = lex_peek;
+    croak "syntax error: expected start of block '{' but found '$c'" unless $c eq '{';
+    {
         local $CAPRPK::{'DEFAULTS::'};
         if ($sigs) {
             lex_read;
@@ -299,16 +308,17 @@ sub parse_body {
             $preamble .= 'my (' . join(', ', @names) . ') = @_;';
 
             for my $name (@names) {
+                my $p;
                 my $s = substr($name,0,1);
                 my $n = substr($name,1);
                 if ($s eq '$') {
-                    my $p = $inv->{name} . '->param("' . $n . '")';
+                    $p = $inv->{name} . '->param("' . $n . '")';
                     $preamble .= $name . ' = ' . $p . ' unless ' . ( $s eq '$' ? 'defined ' : 'scalar ') . $name . ';';
                 }
-                my $p = $inv->{name} . '->query->' . ($s eq '@' ? 'multi_param' : 'param') . '("' . $n . '")';
+                $p = $inv->{name} . '->query->' . ($s eq '@' ? 'multi_param' : 'param') . '("' . $n . '")';
                 $preamble .= $name . ' = ' . $p . ' unless ' . ( $s eq '$' ? 'defined ' : 'scalar ') . $name . ';';
                 if ($s eq '@') {
-                    my $p = $inv->{name} . '->query->' . ($s eq '@' ? 'multi_param' : 'param') . '("' . $n . '[]")';
+                    $p = $inv->{name} . '->query->' . ($s eq '@' ? 'multi_param' : 'param') . '("' . $n . '[]")';
                     $preamble .= $name . ' = ' . $p . ' unless ' . ( $s eq '$' ? 'defined ' : 'scalar ') . $name . ';';
                 }
             }
@@ -331,9 +341,6 @@ sub parse_body {
         }
         $body = parse_block;
     }
-    else {
-        die "syntax error";
-    }
     return $body;
 }
 
@@ -498,7 +505,7 @@ You tried to install another startmode. Placeholders are filled with
 =head1 BUGS
 
 Please report any bugs or feature requests to
-C<bug-cgi-application-plugin-runmodedeclare at rt.cpan.org>, or through the web
+C<bug-cgi-application-plugin-runmodeparsekeyword at rt.cpan.org>, or through the web
 interface at
 L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=CGI-Application-Plugin-RunmodeParseKeyword>.
 I will be notified, and then you'll automatically be notified of progress on
